--[[

	This mod will try to promote/demote guild members based on their contribution and current rank.
	
	So, we wanna go through every member, get their total contribution and rank. Promote or Demote as necessary.

]]

-- [[ Variables. ]]

local PROMOTE_NAME = {}
local PROMOTE_XP = {}
local PROMOTE_RANK = {}
local PROMOTE_CLASS = {}
local PromoteCounter = 0
local DEMOTE_NAME = {}
local DEMOTE_XP = {}
local DEMOTE_RANK = {}
local DEMOTE_CLASS = {}
local DemoteCounter = 0
local TotalCounter = 0
local HEADER = "|cff69CCF0SGP|r: "
local version = 1.0

local FramesStuff = {
		["FRAMES"] = {},
		["CHECKBOXES"] = {},
		["EDITBOXES"] = {},
	}
	

-- [[ Functions. ]]

-- OnLogin
function OnLogin()
	GuildRoster()
	if type(SwynGuildPromoterConfig) ~= "table" then
		SwynGuildPromoterConfig = {}
		SwynGuildPromoterConfig.selectedRanks = {}
	end
	if not SwynGuildPromoterConfig.version then
		SwynGuildPromoterConfig = {}
		SwynGuildPromoterConfig.selectedRanks = {}
		SwynGuildPromoterConfig.version = version
		OnLogin()
	end
	loadEverything()
	print(HEADER .. "SwynGuildPromoter loaded. Type /sgp for list of commands.")
end

-- Clear
function clearLists(msg)
	if msg == "All" or msg == "Promote" then
		ClearTable(PROMOTE_NAME)
		ClearTable(PROMOTE_XP)
		ClearTable(PROMOTE_RANK)
		ClearTable(PROMOTE_CLASS)
		PromoteCounter = 0
	end
	if msg == "All" or msg == "Demote" then
		ClearTable(DEMOTE_NAME)
		ClearTable(DEMOTE_XP)
		ClearTable(DEMOTE_RANK)
		ClearTable(DEMOTE_CLASS)
		DemoteCounter = 0
	end
	TotalCounter = 0
end

-- Clear Table
function ClearTable(TABLE)
	for i,v in pairs(TABLE) do
		TABLE[i] = nil
	end
end

----===================================----
--[[ Snoops on guildies for their info ]]--
----===================================----
function cycleThroughMembers()
	GuildRoster()
	for i = 1,GetNumGuildMembers() do
		local weeklyXP, totalXP = GetGuildRosterContribution(i)
		local name, rank, rankIndex, level, class  = GetGuildRosterInfo(i)
		-- Ignore if higher rank than Stoneguard.
		if rankIndex >= 4 then
			local ModType = rankModifierCalculator(rankIndex+1, rank, totalXP)
			if ModType ~= false then
				if ModType == 1 then
					table.insert(PROMOTE_NAME, name)
					table.insert(PROMOTE_XP, totalXP)
					table.insert(PROMOTE_RANK, rank)
					table.insert(PROMOTE_CLASS, class)
					PromoteCounter = PromoteCounter + 1
				elseif ModType == -1 then
					table.insert(DEMOTE_NAME, name)
					table.insert(DEMOTE_XP, totalXP)
					table.insert(DEMOTE_RANK, rank)
					table.insert(DEMOTE_CLASS, class)
					DemoteCounter = DemoteCounter + 1
				end
			end
		end
	end
end

----==========================----
--[[ Rank Modifier Calculator ]]--
----==========================----
-- Works out if we have to mod the rank, and in which direction.
-- Returns -1 to demote. 0 to do nothing. 1 to promote.
function rankModifierCalculator(rankNum, rankName, XP)
	local RETURN
	local numRanks = GuildControlGetNumRanks()
	
	-- Firstly, are we allowed to mod current rank?
	if FramesStuff["CHECKBOXES"][rankName]:GetChecked() then
		-- Work out our current MinXP and next MinXP
		local currRank_MinXP = tonumber(FramesStuff["EDITBOXES"][rankName]:GetText())
		local nextRank_MinXP = tonumber(FramesStuff["EDITBOXES"][GuildControlGetRankName(rankNum-1)]:GetText())
		if not nextRank_MinXP then
			nextRank_MinXP = -1
		end
		
		if nextRank_MinXP == -1 then
			-- We don't need to promote as next rank has no limit.
			return 0
		elseif XP < currRank_MinXP then
			-- We need to Demote.
			return -1
		elseif XP >= nextRank_MinXP then
			-- We need to promote.
			return 1
		elseif XP >= currRank_MinXP and XP < nextRank_MinXP then
			-- They're within bounds, so do nothing.
			return 0
		else
			print("WTF?!    " .. currRank_MinXP .. " - " .. XP .. " - " .. nextRank_MinXP)
		end
	end
	
	return false
end

----=====================================----
--[[ Promote Members that need Promoting ]]--
----=====================================----
function PromoteMembers()
	print(HEADER .. "Promoting all under-ranked members.")
	for i,v in pairs(PROMOTE_NAME) do
		if CanGuildPromote(v) then
			GuildPromote(v)
		else
			print(HEADER .. "|cffC41F3BFailed|r to promote " .. v .. ".")
		end
	end
	clearLists("Promote")
end

----===================================----
--[[ Demote Members that need Demoting ]]--
----===================================----
function DemoteMembers()
	print(HEADER .. "Demoting all over-ranked members.")
	for i,v in pairs(DEMOTE_NAME) do
		if CanGuildDemote(v) then
			GuildDemote(v)
		else
			print(HEADER .. "|cffC41F3BFailed|r to demote " .. v .. ".")
		end
	end
	clearLists("Demote")
end

----=================================----
--[[ List Showing - Purely for Debug ]]--
----=================================----
-- Show Promote List
function showPromoteList()
	print(HEADER .. "  Promotion List:")
	for i=1,PromoteCounter do
		print(HEADER .. "    Name: |cff" .. getClassColor(PROMOTE_CLASS[i]) .. PROMOTE_NAME[i] .. "|r -> Rank: " .. PROMOTE_RANK[i] .. " -> XP: " .. PROMOTE_XP[i] .. ".")
	end
end
-- Show Demote List
function showDemoteList()
	print(HEADER .. "  Demotion List:")
	for i=1,DemoteCounter do
		print(HEADER .. "    Name: |cff" .. getClassColor(DEMOTE_CLASS[i]) .. DEMOTE_NAME[i] .. "|r -> Rank: " .. DEMOTE_RANK[i] .. " -> XP: " .. DEMOTE_XP[i] .. ".")
	end
end

----==============================----
--[[ Simple class colour function ]]--
----==============================----
function getClassColor(class)
	if class == "Death Knight" then
		return "C41F3B" 
	elseif class == "Druid" then
		return "FF7D0A"
	elseif class == "Hunter" then
		return "ABD473"
	elseif class == "Mage" then
		return "69CCF0"
	elseif class == "Paladin" then
		return "F58CBA"
	elseif class == "Priest" then
		return "FFFFFF"
	elseif class == "Rogue" then
		return "FFF569"
	elseif class == "Shaman" then
		return "0070DE"
	elseif class == "Warlock" then
		return "9482C9"
	elseif class == "Warrior" then
		return "C79C6E"
	else
		return "857C7C"
	end
end


----====================================================================----
--[[ Find and generate a list of guildies that need their rank changing ]]--
----====================================================================----
function Find()
	print(HEADER .. "Finding members who need their ranks changed.")
	clearLists("All")
	cycleThroughMembers()
	print(HEADER .. "Results:")
	print(HEADER .. "  Promotions: |cff3adf38" .. PromoteCounter .. "|r members need to be promoted.")
	showPromoteList()
	print(HEADER .. "  Demotions:  |cffC41F3B" .. DemoteCounter .. "|r members need to be demoted.")
	showDemoteList()
end

----=====================----
--[[ Slash Command Hooks ]]--
----=====================----
SLASH_SGP1, SLASH_SGP2, SLASH_SGP3 = '/sgp', 'sguildpromote', 'swynguildpromote';
function SlashCmdList.SGP(msg, editbox)
	if msg == 'show' then
		for i,v in pairs(FramesStuff["FRAMES"]) do
			v:Show()
		end
	elseif msg == 'hide' then
		for i,v in pairs(FramesStuff["FRAMES"]) do
			v:Hide()
		end
	else
		print(HEADER .. "Slash commands are:")
		print(HEADER .. "  show - Shows the UI.")
		print(HEADER .. "  hide - Hides the UI.")
	end
end

----===============----
--[[ Save Settings ]]--
----===============----
function SaveSettings()
	for i,v in pairs(SwynGuildPromoterConfig.selectedRanks) do
		v = nil
	end
	-- Rank Index
	local numRanks = GuildControlGetNumRanks()
	local ranksTable = {}
	for i=1,numRanks do
		ranksTable[GuildControlGetRankName(i)] = i
	end
	print(HEADER .. "Saving Settings.")
	for i,v in pairs(FramesStuff["CHECKBOXES"]) do
		local name = v:GetName()
		local index = ranksTable[name]
		local ticked = v:GetChecked()
		local limiter = tonumber(FramesStuff["EDITBOXES"][name]:GetText())
		if not limiter then limiter = -1 end
		SwynGuildPromoterConfig.selectedRanks[name] = {name, index, limiter}
		if limiter >= 0 then
			print(HEADER .. name .. " now requires a minimum of " .. limiter .. " activity.")
		end
	end
	print(HEADER .. "Settings Saved.")
end

-- FRAMES STUFF
----========================----
--[[ Creates the Main Frame ]]--
----========================----
function CreateTheMainFrame(name)
	local f = CreateFrame("Frame", name, UIParent)
	f:SetMovable(true)
	f:EnableMouse(true)
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", f.StartMoving)
	f:SetScript("OnDragStop", f.StopMovingOrSizing)
	f:SetSize(400, 360)
	f:SetFrameStrata("MEDIUM")
	f:SetBackdrop({bgFile = "Interface\\ChatFrame\\ChatFrameBackground", 
								edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border", 
								tile = true,
								tileSize = 4,
								edgeSize = 16,
								insets = { 10, 10, 10, 10}})
	f:SetBackdropColor(0.1,0.1,0.1,0.9)
	f:SetBackdropBorderColor(1,1,1,0.5)
	f:SetPoint("CENTER", 0, 0)
		
	local TitleFont = f:CreateFontString(nil,"OVERLAY","GameFontHighlightLarge")
	TitleFont:SetText("Swyn Guild Promoter")
	TitleFont:SetPoint("TOP", f, "TOP", 0, -20)
		
	local FooterFont = f:CreateFontString(nil,"OVERLAY","GameFontHighlight")
	FooterFont:SetText("-- Swyn")
	FooterFont:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -20, 20)
	f:Hide()
	
	local desc = f:CreateFontString(nil,"OVERLAY","GameFontHighlightSmall")
	desc:SetText("Choose which ranks can be promoted or demoted and their limits.\nThen click Find, followed by the action you wish to take.")
	desc:SetPoint("CENTER", f, "CENTER", 0, 115)

	local close = CreateFrame("Button",nil,f,"UIPanelCloseButton")
	close:SetPoint("TOPRIGHT",f,"TOPRIGHT",-4,-4)
	
	FramesStuff["FRAMES"][name] = f
end

----=================----
--[[ Creates Buttons ]]--
----=================----
function CreateButton(name, parent, width, height, text, offsetX, offsetY)
	local b = CreateFrame("Button", name, parent, "UIPanelButtonTemplate")
	b:SetWidth(width)
	b:SetHeight(height)
	b:SetPoint("CENTER", parent, "CENTER", offsetX, offsetY)
	
	--[[Button Scripts]]--
	-- Save Settings
	if name == "SwynGuildPromote_Button_Save" then
		b:SetScript("OnClick", SaveSettings)
	end
	-- Find --
	if name == "SwynGuildPromote_Button_Find" then
		b:SetScript("OnClick", Find)
	end
	-- Promote and Demote
	if name == "SwynGuildPromote_Button_DoBoth" then
		b:SetScript("OnClick", function()
						PromoteMembers()
						DemoteMembers()
					end)
	end
	-- Promote
	if name == "SwynGuildPromote_Button_OnlyPromote" then
		b:SetScript("OnClick", PromoteMembers)
	end
	-- Demote
	if name == "SwynGuildPromote_Button_OnlyDemote" then
		b:SetScript("OnClick", DemoteMembers)
	end
	
	local font = b:CreateFontString(nil,"OVERLAY","GameFontHighlight")
	font:SetText(text)
	font:SetPoint("CENTER", parent, "CENTER", offsetX, offsetY)
	FramesStuff["FRAMES"][name] = b
end

----=====================----
--[[ Creates Check Boxes ]]--
----=====================----
function CreateCheckbox(name, parent, text, offsetX, offsetY)
	local cb = CreateFrame("CheckButton", name, parent, "OptionsBaseCheckButtonTemplate")
	cb:SetPoint("CENTER", parent, "CENTER", offsetX, offsetY)
	
	local font = cb:CreateFontString("CheckFont"..name, "OVERLAY", "GameFontHighlight")
	font:SetText(text)
	font:SetPoint("LEFT", cb, "RIGHT", 4, 1)
	
	FramesStuff["CHECKBOXES"][name] = cb
end

----=============================================----
--[[ Creates the checkboxes based on guild ranks ]]--
----=============================================----
function CreateRankCheckboxes(parent)
	-- Title
	local title = parent:CreateFontString(nil,"OVERLAY","GameFontHighlight")
	title:SetText("Ranks")
	title:SetPoint("CENTER", parent, "CENTER", 30, 80)
	-- First work out how many guild ranks there are.
	GuildRoster()
	local numRanks = GuildControlGetNumRanks()
	local offsetX = -5
	local offsetY = 50
	
	for i=2,numRanks do
		local rankName = GuildControlGetRankName(i)
		CreateCheckbox(rankName, parent, rankName, offsetX, offsetY-(25*(i-2)))
		if SwynGuildPromoterConfig.selectedRanks[rankName] and SwynGuildPromoterConfig.selectedRanks[rankName][3] >= 0 then
			FramesStuff["CHECKBOXES"][rankName]:SetChecked(true)
		end
	end
end

----==================----
--[[ Creats editboxes ]]--
----==================----
function CreateEditbox(name, parent, width, height, offsetX, offsetY)
	local eb = CreateFrame("EditBox", name, parent, "InputBoxTemplate")
	eb:SetPoint("CENTER", parent, "CENTER", offsetX, offsetY)
	eb:SetMaxLetters(8)
	eb:SetWidth(width)
	eb:SetHeight(height)
	eb:SetAutoFocus(false)
	eb:SetScript("OnEnterPressed", function(self) eb:ClearFocus() end)
	
	FramesStuff["EDITBOXES"][name] = eb
end

----=============================================----
--[[ Creates the editboxes to be used for limits ]]--
----=============================================----
function CreateRankEditboxes(parent)
	-- Title
	local title = parent:CreateFontString(nil,"OVERLAY","GameFontHighlight")
	title:SetText("Lower Limits")
	title:SetPoint("CENTER", parent, "CENTER", 145, 80)
	-- First work out how many guild ranks there are.
	GuildRoster()
	local numRanks = GuildControlGetNumRanks()
	local offsetX = 140
	local offsetY = 52
	
	for i=2,numRanks do
		local rankName = GuildControlGetRankName(i)
		CreateEditbox(rankName, parent, 60, 25, offsetX, offsetY-(25*(i-2)))
		if SwynGuildPromoterConfig.selectedRanks[rankName] and SwynGuildPromoterConfig.selectedRanks[rankName][3] >= 0 then
			FramesStuff["EDITBOXES"][rankName]:SetText(SwynGuildPromoterConfig.selectedRanks[rankName][3])
		else
			FramesStuff["EDITBOXES"][rankName]:SetText("")
		end
		if i == numRanks then
			FramesStuff["EDITBOXES"][rankName]:SetText(0)
			FramesStuff["EDITBOXES"][rankName]:Hide()
		end
	end
end

----==============================----
--[[ Actually Generate the Frames ]]--
----==============================----
local SGP_FRAME = CreateFrame("Frame")
SGP_FRAME:RegisterEvent("PLAYER_LOGIN")
SGP_FRAME:SetScript("OnEvent",OnLogin)

function loadEverything()
	CreateTheMainFrame("SwynGuildPromote_Frame")
	CreateButton("SwynGuildPromote_Button_Find", FramesStuff["FRAMES"]["SwynGuildPromote_Frame"], 160, 30, "Find", -105, 60)
	CreateButton("SwynGuildPromote_Button_DoBoth", FramesStuff["FRAMES"]["SwynGuildPromote_Frame"], 160, 30, "Promote and Demote", -105, 0)
	CreateButton("SwynGuildPromote_Button_OnlyPromote", FramesStuff["FRAMES"]["SwynGuildPromote_Frame"], 160, 30, "Only Promote", -105, -40)
	CreateButton("SwynGuildPromote_Button_OnlyDemote", FramesStuff["FRAMES"]["SwynGuildPromote_Frame"], 160, 30, "Only Demote", -105, -80)
	CreateRankCheckboxes(FramesStuff["FRAMES"]["SwynGuildPromote_Frame"])
	CreateRankEditboxes(FramesStuff["FRAMES"]["SwynGuildPromote_Frame"])
	CreateButton("SwynGuildPromote_Button_Save", FramesStuff["FRAMES"]["SwynGuildPromote_Frame"], 100, 30, "Save Settings", 50, -135)
end